/* Copyright � 2004 Ted Phelps */

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include "freadnums.h"

#define BUFFER_SIZE 512

#define ST_START 0
#define ST_NUM 1
#define ST_COMMA 2
#define ST_CR 3

int freadnums(FILE *file, double *values, int value_count)
{
    char buffer[BUFFER_SIZE];
    char *end;
    int state = ST_START;
    int count = 0;
    double value;
    int ch;
    int i = 0;

    while (1) {
        /* Read the next character */
        if ((ch = fgetc(file)) == EOF && ferror(file)) {
            return -1;
        }

        switch (state)
        {
            case ST_START:
                if (ch == EOF || ch == '\n') {
                    return count;
                } else if (ch == '\r') {
                    state = ST_CR;
                } else if (ch == ',') {
                    state = ST_COMMA;
                } else if (!isspace(ch)) {
                    i = 0;
                    buffer[i++] = ch;
                    state = ST_NUM;
                }

                break;

            case ST_NUM:
                /* Watch for the end of our number */
                if (ch == EOF || ch == ',' || isspace(ch)) {
                    /* Convert the number to a double */
                    buffer[i] = '\0';
                    value = strtod(buffer, &end);
                    if (*end != '\0') {
                        return -1;
                    }

                    /* Record it if there's room */
                    if (count < value_count) {
                        values[count] = value;
                    }

		    /* Count it even if there isn't room */
		    count++;

                    /* Cope with the new character */
                    if (ch == EOF || ch == '\n') {
                        return count;
                    } else if (ch == '\r') {
                        state = ST_CR;
                    } else {
                        state = ST_COMMA;
                    }
                } else {
                    if (i < BUFFER_SIZE) {
                        buffer[i++] = ch;
                    }
                }

                break;

            case ST_COMMA:
                if (ch == EOF || ch == '\n') {
                    return count;
                } else if (ch == '\r') {
                    state = ST_CR;
                } else if (ch == ',') {
                    return -1;
                } else if (!isspace(ch)) {
                    i = 0;
                    buffer[i++] = ch;
                    state = ST_NUM;
                }

                break;

            case ST_CR:
                /* If the next character is not a LF or EOF then push it back */
                if (ch != EOF && ch != '\n') {
                    if ((ungetc(ch, file)) == EOF) {
                        abort();
                    }
                }

                return count;
        }
    }
}

#if 0
int main(int argc, char *argv[])
{
    double values[16];
    int result, i;

    while (1)
    {
        printf("freadnums: "); fflush(stdout);
        result = freadnums(stdin, values, 16);

        if (result == 0) {
            exit(0);
        }

        if (result < 0) {
            fprintf(stderr, "error\n");
            exit(1);
        }

        printf("%g", values[0]);
        for (i = 1; i < result; i++) {
            printf(", %g", values[i]);
        }
        printf("\n");
    }
}
#endif
